@@ -7,14 +7,16 @@ var InlineSpellCheckerUI = {
   mSpellSuggestions: [], // text of words
   mSuggestionItems: [],  // menuitem nodes
   mDictionaryMenu: null,
-  mDictionaryNames: [],
+  mDictionaryNames: [], // internal dictionary names (e.g. "en-US")
   mDictionaryItems: [],
+  mLanguageBundle: null, // language names
+  mRegionBundle: null, // region names
 
-  // Call this function to initialize for a given edit element
-  init: function(inputElt)
+  // Call this function to initialize for a given editor
+  init: function(aEditor)
   {
     this.uninit();
-    this.mEditor = inputElt.QueryInterface(Components.interfaces.nsIDOMNSEditableElement).editor;
+    this.mEditor = aEditor;
     try {
       this.mInlineSpellChecker = this.mEditor.inlineSpellChecker;
       // note: this might have been NULL if there is no chance we can spellcheck
@@ -80,7 +82,7 @@ var InlineSpellCheckerUI = {
   set enabled(isEnabled)
   {
     if (this.mInlineSpellChecker)
-      this.mInlineSpellChecker.enableRealTimeSpell = isEnabled;
+      this.mEditor.QueryInterface(Components.interfaces.nsIEditor_MOZILLA_1_8_BRANCH).setSpellcheckUserOverride(isEnabled);
   },
 
   // returns true if the given event is over a misspelled word
@@ -141,6 +143,16 @@ var InlineSpellCheckerUI = {
     this.mDictionaryNames = [];
     this.mDictionaryItems = [];
 
+    if (! this.mLanguageBundle) {
+      // create the bundles for language and region
+      var bundleService = Components.classes["@mozilla.org/intl/stringbundle;1"]
+                                    .getService(Components.interfaces.nsIStringBundleService);
+      this.mLanguageBundle = bundleService.createBundle(
+          "chrome://global/locale/languageNames.properties");
+      this.mRegionBundle = bundleService.createBundle(
+          "chrome://global/locale/regionNames.properties");
+    }
+
     if (! this.mInlineSpellChecker || ! this.enabled)
       return 0;
     var spellchecker = this.mInlineSpellChecker.spellChecker;
@@ -149,11 +161,30 @@ var InlineSpellCheckerUI = {
     var list = o1.value;
     var listcount = o2.value;
     var curlang = spellchecker.GetCurrentDictionary();
+    var isoStrArray;
 
     for (var i = 0; i < list.length; i ++) {
+      // get the display name for this dictionary
+      var isoStrArray = list[i].split("-");
+      var displayName = "";
+      if (this.mLanguageBundle && isoStrArray[0]) {
+        try {
+          displayName = this.mLanguageBundle.GetStringFromName(isoStrArray[0].toLowerCase());
+        } catch(e) {} // ignore language bundle errors
+        if (this.mRegionBundle && isoStrArray[1]) {
+          try {
+            displayName += " / " + this.mRegionBundle.GetStringFromName(isoStrArray[1].toLowerCase());
+          } catch(e) {} // ignore region bundle errors
+        }
+      }
+
+      // if we didn't get a name, just use the raw dictionary name
+      if (displayName.length == 0)
+        displayName = list[i];
+
       this.mDictionaryNames.push(list[i]);
       var item = document.createElement("menuitem");
-      item.setAttribute("label", list[i]);
+      item.setAttribute("label", displayName);
       item.setAttribute("type", "checkbox");
       this.mDictionaryItems.push(item);
       if (curlang == list[i]) {
@@ -185,7 +216,10 @@ var InlineSpellCheckerUI = {
   {
     if (! this.mInlineSpellChecker || index < 0 || index >= this.mDictionaryNames.length)
       return;
-    this.mInlineSpellChecker.spellChecker.SetCurrentDictionary(this.mDictionaryNames[index]);
+    var spellchecker = this.mInlineSpellChecker.spellChecker;
+    spellchecker.QueryInterface(Components.interfaces.nsIEditorSpellCheck_MOZILLA_1_8_BRANCH);
+    spellchecker.SetCurrentDictionary(this.mDictionaryNames[index]);
+    spellchecker.saveDefaultDictionary();
     this.mInlineSpellChecker.spellCheckRange(null); // causes recheck
   },
 
@@ -203,8 +237,7 @@ var InlineSpellCheckerUI = {
   // callback for enabling or disabling spellchecking
   toggleEnabled: function()
   {
-    this.mInlineSpellChecker.enableRealTimeSpell =
-        ! this.mInlineSpellChecker.enableRealTimeSpell;
+    this.mEditor.QueryInterface(Components.interfaces.nsIEditor_MOZILLA_1_8_BRANCH).setSpellcheckUserOverride(!this.mInlineSpellChecker.enableRealTimeSpell);
   },
 
   // callback for adding the current misspelling to the user-defined dictionary
