@@ -1,4 +1,4 @@
-//@line 39 "/builds/tinderbox/XR-Mozilla1.8.0/Linux_2.4.21-32.0.1.ELsmp_Depend/mozilla/toolkit/mozapps/downloads/content/DownloadProgressListener.js"
+//@line 39 "/builds/tinderbox/XR-Mozilla1.8/Linux_2.4.21-32.0.1.ELsmp_Depend/mozilla/toolkit/mozapps/downloads/content/DownloadProgressListener.js"
 
 var gStrings = [];
 const interval = 500; // Update every 500 milliseconds.
@@ -22,7 +22,6 @@ function DownloadProgressListener (aDocument, aStringBundle)
 
 DownloadProgressListener.prototype = 
 {
-  elapsed: 0,
   rateChanges: 0,
   rateChangeLimit: 0,
   priorRate: 0,
@@ -54,14 +53,6 @@ DownloadProgressListener.prototype =
     // Update this time.
     this.lastUpdate = now;
 
-    // Update download rate.
-    this.elapsed = now - (aDownload.startTime / 1000);
-    var rate; // aCurTotalProgress/sec
-    if (this.elapsed)
-      rate = (aCurTotalProgress * 1024) / this.elapsed;
-    else
-      rate = 0;
-
     var aDownloadID = aDownload.targetFile.path;
     var download = this.doc.getElementById(aDownloadID);
 
@@ -101,6 +92,7 @@ DownloadProgressListener.prototype =
     if (download)
       download.setAttribute("status-internal", kbProgress);
 
+    var rate = aDownload.QueryInterface(Components.interfaces.nsIDownload_MOZILLA_1_8_BRANCH).speed;
     if (rate) {
       // rate is bytes/sec
       var kRate = rate / 1024; // K bytes/sec;
@@ -120,13 +112,13 @@ DownloadProgressListener.prototype =
       else
         this.rateChanges = 0;
 
-        var fraction = kRate % 10;
-        kRate = parseInt((kRate - fraction) / 10);
+      var fraction = kRate % 10;
+      kRate = parseInt((kRate - fraction) / 10);
 
-        // Insert 3 is the download rate (in kilobytes/sec).
-        if (kRate < 100)
-          kRate += "." + fraction;
-        status = this._replaceInsert(status, 2, kRate);
+      // Insert 3 is the download rate (in kilobytes/sec).
+      if (kRate < 100)
+        kRate += "." + fraction;
+      status = this._replaceInsert(status, 2, kRate);
     }
     else
       status = this._replaceInsert(status, 2, "??.?");
